#include <fat.h>
#include <nds.h>
#include <nds/arm9/console.h>
#include <stdio.h>
#include <math.h>

#include <ipc2.h>

#define POWER_OFF_VBLANKS 600

#include "screen.h"
#include "sound.h"
#include "playlist.h"

extern void displayconsole(void);
extern void displayconsoleinit(void);
extern void initdisplays(void);

int main(void) {
	unsigned power_state = 0, power_vblanks = POWER_OFF_VBLANKS;
	int lastkeys;

	powerON(POWER_ALL_2D);

	irqInit();
	irqEnable(IRQ_VBLANK);

	screen_initdisplays();

	fatInitDefault();

	IPC2->sound_control = 0;
	IPC2->sound_mode = IPC2_SOUND_MODE_PCM_NORMAL;

	file_scan("/");

	file_sort();

	playlist_add(playlist_build_songs());

	file_scan_playlists("/");

	currentmedia = media;

	screen_mainmenu();

	state = STOPPED;

	IPC2->messageflag = IPC2->messageflag2 = 0;

	while(1) {
		displayconsole();

		scanKeys();

		if((keysDown() & KEY_START) != 0) {
			static enum { CONSOLE, GRAPHICS } mode = GRAPHICS;

			if(mode == GRAPHICS) {
				videoSetMode(MODE_5_2D | DISPLAY_BG0_ACTIVE);
				mode = CONSOLE;
			} else {
				videoSetMode(MODE_5_2D | DISPLAY_BG2_ACTIVE | DISPLAY_BG3_ACTIVE
					| DISPLAY_SPR_ACTIVE | DISPLAY_SPR_1D_LAYOUT | DISPLAY_SPR_1D_BMP);
				mode = GRAPHICS;
			}
		}
		if((keysDown() & KEY_SELECT) != 0) {
			static enum { NORMAL, OV2x, OV4x } playmode = NORMAL;;

			if(playmode == NORMAL) {
				IPC2->sound_mode = IPC2_SOUND_MODE_PCM_NORMAL;
				playmode = OV2x;
			} else if(playmode == OV2x) {
				IPC2->sound_mode = IPC2_SOUND_MODE_PCM_OVERSAMPLING2x;
				playmode = OV4x;
			} else if(playmode == OV4x) {
				IPC2->sound_mode = IPC2_SOUND_MODE_PCM_OVERSAMPLING4x;
				playmode = NORMAL;
			}
		}

		screen_update();
//		sound_update();

		if(keysHeld() != lastkeys) {
			lastkeys = keysHeld();
			power_vblanks = POWER_OFF_VBLANKS;
		} else if(power_vblanks > 0)
			power_vblanks--;

		if((power_vblanks == 0 || (keysHeld() & KEY_LID)) && power_state == 0) {
			power_state = 1;

			IPC_SendSync(IPC2_REQUEST_SET_BACKLIGHTS_OFF);
			swiWaitForVBlank(); /* HACK */
			IPC_SendSync(IPC2_REQUEST_LEDBLINK_ON);
			swiWaitForVBlank(); /* HACK */
		} else if(power_vblanks > 0 && (keysHeld() & KEY_LID) == 0 && power_state == 1) {
			power_state = 0;

			IPC_SendSync(IPC2_REQUEST_SET_BACKLIGHTS_ON);
			swiWaitForVBlank(); /* HACK */
			IPC_SendSync(IPC2_REQUEST_LEDBLINK_OFF);
			swiWaitForVBlank(); /* HACK */
		}

		swiWaitForVBlank();
	}

	return 0;
}
