#include <tremor/ivorbiscodec.h>
#include <tremor/ivorbisfile.h>

#include <nds.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <ipc2.h>

#define SOUND_NEED_BUFFER

#include "sound.h"
#include "file.h"

#define TREMOR_SAMPLES 1152

char *readbuffer;
static OggVorbis_File vf;
static int channels;
static int current_section;
static int update;

void wait() {
	int i;
	for(i=0;i<120;i++)
		swiWaitForVBlank();
}
void readtremor(void) {
	int readbytes;
	int ret;
	static int call = 0;

	if(state == FINISHING)
		return;

	if(channels == 1) {
		ret = ov_read(&vf, pcmL, TREMOR_SAMPLES * 2 * channels, &current_section);

		if(ret <= 0) {
			state = FINISHING;
			return;
		}

		readbytes += ret;

		while(readbytes < TREMOR_SAMPLES * 2 * channels) {
			ret = ov_read(&vf, ((char *) pcmL) + readbytes, TREMOR_SAMPLES * 2 * channels - readbytes, &current_section);

			if(ret <= 0) {
				state = FINISHING;
				return;
			}

			readbytes += ret;
		}
	}

	else if(channels == 2) {
		int i;
		s16 *src, *dst;

		readbytes = 0;

		ret = ov_read(&vf, readbuffer, TREMOR_SAMPLES * 2 * channels, &current_section);

		if(ret <= 0) {
			state = FINISHING;
			return;
		}

		readbytes += ret;

		while(readbytes < TREMOR_SAMPLES * 2 * channels) {
			ret = ov_read(&vf, ((char *) readbuffer) + readbytes, TREMOR_SAMPLES * 2 * channels - readbytes, &current_section);

			if(ret <= 0) {
				state = FINISHING;
				return;
			}

			readbytes += ret;
		}

		src = (s16 *) readbuffer;
		dst = (s16 *) pcmL;
		for(i = 0; i < 2 * TREMOR_SAMPLES; i += 2)
			dst[i/2] = src[i];

		src = (s16 *) (((s16 *)readbuffer) + 1);
		dst = (s16 *) pcmR;
		for(i = 0; i < 2 * TREMOR_SAMPLES; i += 2)
			dst[i/2] = src[i];
	}

	IPC2->sound_writerequest = 0;
}

void InterruptHandler_IPC_SYNC_tremor(void) {
	u8 sync; 

	sync = IPC_GetSync();

	if(sync == IPC2_REQUEST_WRITE_SOUND) {
		switch(format) {
			case TREMOR:
				update = 1;
				if(state != PLAYING)
					IPC2->sound_writerequest = 0;
				/* for some reason, this code doesn't work when the function is
				 * called within an interrupt :-/ */
//				readtremor();
//				DC_FlushRange(pcmL, TREMOR_SAMPLES * 2);
//				DC_FlushRange(pcmR, TREMOR_SAMPLES * 2);
				break;

			default:
				break;
		}
	}
}

void tremor_update(void) {
	int old = REG_IME;

	REG_IME = 0;
	if(update == 1) {
		readtremor();
		DC_FlushRange(pcmL, TREMOR_SAMPLES * 2);
		DC_FlushRange(pcmR, TREMOR_SAMPLES * 2);

		update = 0;
	}

	REG_IME = old;
}

int tremor_play(FILE *fp_arg) {
	vorbis_info *vi;
	char **ptr, *s;
	long len;
	int ret;

//	iprintf("\x1b[11;0Htremor_play()");
//	wait();

//	readbuffer = sound_buffer;
	readbuffer = (char*) malloc(8192);
	format = TREMOR;

	ov_open(fp_arg, &vf, NULL, 0);

	current_section = 0;

	ptr = ov_comment(&vf,-1)->user_comments;
	vi = ov_info(&vf,-1);
	while(*ptr)
		++ptr;
	len = ov_pcm_total(&vf,-1);
	s = ov_comment(&vf,-1)->vendor;

	IPC2->sound_channels = channels = vi->channels;
	IPC2->sound_frequency = vi->rate;
	IPC2->sound_bytes_per_sample = 2;
	IPC2->sound_samples = TREMOR_SAMPLES;
	IPC2->sound_writerequest = 0;

	IPC2->sound_lbuf = (void *) pcmL;
	IPC2->sound_rbuf = (void *) pcmR;

	irqSet(IRQ_IPC_SYNC, InterruptHandler_IPC_SYNC_tremor);
	REG_IPC_SYNC = IPC_SYNC_IRQ_ENABLE;

	return 0;
}

void tremor_playpause(void) {
	if(state == PLAYING)
		IPC_SendSync(IPC2_REQUEST_STOP_PLAYING);
	else if(state == STOPPED || state == PAUSED);
		IPC2->sound_control = IPC2_SOUND_START;
}

void tremor_stop(void) {
	ov_clear(&vf);
}
