#include <nds.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <unistd.h>

#include <ipc2.h>

#define SOUND_NEED_BUFFER

#include "file.h"
#include "sound.h"

static char *readbuffer;
static struct waveheader waveheader;
static u16 samples = 1152;

static void readwave() {
	int readsamples = samples;

	if(state == FINISHING)
		return;

	if(ftell(sound_playfile) + readsamples * waveheader.fmt_channels * waveheader.fmt_bitspersample / 8 > waveheader.data_size) {
		readsamples = (waveheader.data_size - ftell(sound_playfile)) / (waveheader.fmt_channels * waveheader.fmt_bitspersample / 8);

		memset(pcmL, 0, samples * waveheader.fmt_bitspersample / 8);
		memset(pcmR, 0, samples * waveheader.fmt_bitspersample / 8);

		state = FINISHING;
	}

	if(waveheader.fmt_channels == 1)
		fread(pcmL, waveheader.fmt_bitspersample / 8, readsamples, sound_playfile);

	else if(waveheader.fmt_channels == 2) {
		int i;

		if(waveheader.fmt_bitspersample == 8) {
			s8 *src, *dst;

			fread(readbuffer, 2, readsamples, sound_playfile);

			src = (s8 *) readbuffer;
			dst = (s8 *) pcmL;
			for(i = 0; i < 2 * readsamples; i += 2)
				dst[i/2] = src[i];

			src = (s8 *) (((s8 *) (readbuffer)) + 1);
			dst = (s8 *) pcmR;
			for(i = 0; i < 2 * readsamples; i += 2)
				dst[i/2] = src[i];
		} else if(waveheader.fmt_bitspersample == 16) {
			s16 *src, *dst;

			fread(readbuffer, 2 * 2, readsamples, sound_playfile);

			src = (s16 *) readbuffer;
			dst = (s16 *) pcmL;
			for(i = 0; i < 2 * readsamples; i += 2)
				dst[i/2] = src[i];

			src = (s16 *) (((s16 *)readbuffer) + 1);
			dst = (s16 *) pcmR;
			for(i = 0; i < 2 * readsamples; i += 2)
				dst[i/2] = src[i];
		}
	}

	IPC2->sound_writerequest = 0;
}

static void InterruptHandler_IPC_SYNC_wave(void) {
	u8 sync;

	sync = IPC_GetSync();

	if(sync == IPC2_REQUEST_WRITE_SOUND) {
		switch(format) {
			case WAVE:
				readwave();
				DC_FlushRange(pcmL, samples * waveheader.fmt_bitspersample / 8);
				DC_FlushRange(pcmR, samples * waveheader.fmt_bitspersample / 8);
				break;

			default:
				break;
		}
	}
}

int waveplay(FILE *fp_arg) {
	/* TODO: code to detect corrupted files */
	sound_playfile = fp_arg;
	format = WAVE;

	readbuffer = sound_buffer;
	fread(&waveheader, 44, 1, sound_playfile);

	IPC2->sound_channels = waveheader.fmt_channels;
	IPC2->sound_frequency = waveheader.fmt_samplerate;
	IPC2->sound_bytes_per_sample = waveheader.fmt_bitspersample / 8;
	IPC2->sound_samples = samples;
	IPC2->sound_writerequest = 0;

	IPC2->sound_lbuf = (void *) pcmL;
	IPC2->sound_rbuf = (void *) pcmR;

	irqSet(IRQ_IPC_SYNC, InterruptHandler_IPC_SYNC_wave);
	REG_IPC_SYNC = IPC_SYNC_IRQ_ENABLE;

	return 0;
}

static int wavecontinue(FILE *fp_arg) {
	IPC2->sound_lbuf = (void *) pcmL;
	IPC2->sound_rbuf = (void *) pcmR;

	irqSet(IRQ_IPC_SYNC, InterruptHandler_IPC_SYNC_wave);
	REG_IPC_SYNC = IPC_SYNC_IRQ_ENABLE;

	IPC2->sound_control = IPC2_SOUND_START;

	return 0;
}

void wave_stop(void) {
}

void wave_playpause(void) {
	if(state == PLAYING)
		IPC_SendSync(IPC2_REQUEST_STOP_PLAYING);
	else if(state == STOPPED || state == PAUSED)
		IPC2->sound_control = IPC2_SOUND_START;
}

void wave_rewind(void) {
	if(ftell(sound_playfile) < waveheader.fmt_byterate)
		return;

	sound_stop();
	fseek(sound_playfile, -waveheader.fmt_byterate, SEEK_CUR);
	wavecontinue(sound_playfile);
}

void wave_forward(void) {
	if((ftell(sound_playfile) + waveheader.fmt_byterate) > waveheader.data_size)
		return;

	sound_stop();
	fseek(sound_playfile, waveheader.fmt_byterate, SEEK_CUR);
	wavecontinue(sound_playfile);
}
